from airunner.windows.setup_wizard.model_setup.stable_diffusion_setup.templates.choose_version_ui import Ui_choose_model_version
from airunner.windows.setup_wizard.setup_wizard_window import BaseWizard


class ChooseModelVersion(BaseWizard):
    class_name_ = Ui_choose_model_version

