font_settings_bootstrap_data = (
    {
        "name": "chat",
        "font_family": "Arial",
        "font_size": 16,
    },
    {
        "name": "primary",
        "font_family": "Arial",
        "font_size": 12,
    }
)
